from datetime import datetime as Datetime
from functools import wraps
from pathlib import Path
from typing import Any, Final, cast
from uuid import uuid4

import nest_asyncio
import typer
from colorama import Back, Fore

import beni
import beni.logging as blogging
import beni.print as bprint

_startTime: Datetime | None = None
_logFile: Path | None = None

app: Final = typer.Typer()


def w_task(func: beni.WrappedFunc) -> beni.WrappedFunc:
    @wraps(func)
    def wraper(*args: Any, **kwargs: Any):
        global _startTime
        _startTime = Datetime.now()
        try:
            blogging.init_logger(logFile=_logFile)
            return func(*args, **kwargs)
        except BaseException as ex:
            bprint.set_color(Fore.LIGHTRED_EX)
            blogging.error(str(ex))
            blogging.error('执行失败')
            raise
        finally:

            if blogging.getcount_critical():
                color = Fore.LIGHTWHITE_EX + Back.LIGHTMAGENTA_EX
            elif blogging.getcount_error():
                color = Fore.LIGHTWHITE_EX + Back.LIGHTRED_EX
            elif blogging.getcount_warning():
                color = Fore.BLACK + Back.LIGHTYELLOW_EX
            else:
                color = Fore.BLACK + Back.LIGHTGREEN_EX

            bprint.set_color(color)
            blogging.info('-' * 75)

            msgAry = ['任务结束']
            if blogging.getcount_critical():
                msgAry.append(f'critical({blogging.getcount_critical()})')
            if blogging.getcount_error():
                msgAry.append(f'error({blogging.getcount_error()})')
            if blogging.getcount_warning():
                msgAry.append(f'warning({blogging.getcount_warning()})')

            bprint.set_color(color)
            blogging.info(' '.join(msgAry))

            passTime = str(Datetime.now() - _startTime)
            if passTime.startswith('0:'):
                passTime = '0' + passTime
            blogging.info(f'用时: {passTime}')

    return cast(Any, wraper)


def set_log_path(log_path: Path):
    if _startTime:
        blogging.warning('task.setLogDir 必须在任务启动前调用，本次忽略执行')
    else:
        global _logFile
        _logFile = beni.getpath(log_path, f'{uuid4()}.log')


@w_task
def run():
    nest_asyncio.apply()
    app()


@w_task
def debug(*args: str):
    from typer.testing import CliRunner
    CliRunner().invoke(app, args)
